@@ -10,7 +10,7 @@ use swc_common::{
     chain, comments::Comments, errors::Handler, sync::Lrc, FileName, Mark, SourceMap,
 };
 use swc_ecma_ast::Module;
-use swc_ecma_minifier::option::MinifyOptions;
+use swc_ecma_minifier::{hygiene_optimizer, option::MinifyOptions, unique_scope};
 use swc_ecma_parser::Syntax;
 use swc_ecma_transforms::{
     compat, fixer, helpers, hygiene, hygiene::hygiene_with_config, modules, modules::util::Scope,
@@ -25,10 +25,12 @@ pub struct PassBuilder<'a, 'b, P: swc_ecma_visit::Fold> {
     handler: &'b Handler,
     env: Option<swc_ecma_preset_env::Config>,
     pass: P,
+    /// [Mark] for top level bindings and unresolved identifier references.
     global_mark: Mark,
     target: JscTarget,
     loose: bool,
     hygiene: Option<hygiene::Config>,
+    optimize_hygiene: bool,
     fixer: bool,
     inject_helpers: bool,
     minify: Option<JsMinifyOptions>,
@@ -45,12 +47,13 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         PassBuilder {
             cm,
             handler,
+            env: None,
             pass,
-            target: JscTarget::Es5,
             global_mark,
+            target: JscTarget::Es5,
             loose,
             hygiene: Some(Default::default()),
-            env: None,
+            optimize_hygiene: false,
             fixer: true,
             inject_helpers: true,
             minify: None,
@@ -65,12 +68,13 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         PassBuilder {
             cm: self.cm,
             handler: self.handler,
+            env: self.env,
             pass,
+            global_mark: self.global_mark,
             target: self.target,
             loose: self.loose,
             hygiene: self.hygiene,
-            env: self.env,
-            global_mark: self.global_mark,
+            optimize_hygiene: self.optimize_hygiene,
             fixer: self.fixer,
             inject_helpers: self.inject_helpers,
             minify: self.minify,
@@ -101,6 +105,11 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         self
     }
 
+    pub fn optimize_hygiene(mut self, enable: bool) -> Self {
+        self.optimize_hygiene = enable;
+        self
+    }
+
     pub fn const_modules(
         self,
         globals: HashMap<JsWord, HashMap<JsWord, String>>,
@@ -225,7 +234,13 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 top_level_mark: self.global_mark,
             }),
             Optional::new(
-                hygiene_with_config(self.hygiene.clone().unwrap_or_default()),
+                chain!(
+                    Optional::new(
+                        chain!(unique_scope(), hygiene_optimizer(self.global_mark)),
+                        self.optimize_hygiene
+                    ),
+                    hygiene_with_config(self.hygiene.clone().unwrap_or_default())
+                ),
                 self.hygiene.is_some()
             ),
             Optional::new(fixer(comments.map(|v| v as &dyn Comments)), self.fixer),
