@@ -745,6 +745,7 @@ fn should_visit() {
                 is_module: config.is_module,
                 output_path: config.output_path,
                 source_file_name: config.source_file_name,
+                preserve_comments: config.preserve_comments,
             };
 
             if config.minify {
@@ -771,6 +772,7 @@ fn should_visit() {
                 config.source_maps,
                 None, // TODO: figure out sourcemaps
                 config.minify,
+                config.preserve_comments,
             )
             .unwrap()
             .code)
